﻿
// http://dotnetfollower.com
namespace dotNetFollower
{
    /// <summary>
    /// Represents the awaiting parameters
    /// </summary>
    public class AsyncOperationAwaitingParams
    {
	    private const int DefaultTimeout    = 2000;
	    private const int DefaultRetryCount = 100;

	    private int  _startTimeout          = DefaultTimeout;
	    private int  _retryCount            = DefaultRetryCount;
	    private bool _doNotCountIfSuspended = true;
	    private bool _doNotCountIfPaused    = true;

	    /// <summary>
	    /// If set, specifies the value to be added to the sleep timeout on each iteration
	    /// </summary>
	    public int? TimeoutStep { get; set; }
	    /// <summary>
	    /// If set, specifies the top limit of the sleep timeout
	    /// </summary>
	    public int? EndTimeout  { get; set; }

	    /// <summary>
	    /// Indicates if we need to wait for the operation completeness eternally
	    /// </summary>
	    public bool WaitForever { get; set; }

	    /// <summary>
	    /// Specifies the initial and maximal number of iterations
	    /// </summary>
	    public int RetryCount
	    {
		    get { return _retryCount; }
		    set { _retryCount = value; }
	    }
	    /// <summary>
	    /// Specifies the initial sleep timeout
	    /// </summary>
	    public int StartTimeout
	    {
		    get { return _startTimeout; }
		    set { _startTimeout = value; }
	    }
	    /// <summary>
	    /// Indicates if an iteration counts when the operation is suspended
	    /// </summary>
	    public bool DoNotCountIfSuspended
	    {
		    get { return _doNotCountIfSuspended; }
		    set { _doNotCountIfSuspended = value; }
	    }
	    /// <summary>
	    /// Indicates if an iteration counts when the operation is paused
	    /// </summary>
	    public bool DoNotCountIfPaused
	    {
		    get { return _doNotCountIfPaused; }
		    set { _doNotCountIfPaused = value; }
	    }
    }
}
