﻿using System;

// http://dotnetfollower.com
namespace dotNetFollower
{
    /// <summary>
    /// Represents current state of an operation
    /// </summary>
    /// <typeparam name="TState">Represents a type of the operation state</typeparam>
    /// <typeparam name="TStatus">Represents a type of the operation status</typeparam>
    [Serializable]
    public abstract class AsyncOperationState<TState, TStatus>
    {
        /// <summary>
        /// Indicates how many retries remain
        /// </summary>
        public int  CurrentRetryCount    { get; set; }
        /// <summary>
        /// Indicates sleep timeout on the last iteration
        /// </summary>
        public int  CurrentTimeout       { get; set; }

        /// <summary>
        /// Represents current operation state
        /// </summary>
        public TState  CurrentState      { get; set; }
        /// <summary>
        /// Represents current operation status
        /// </summary>
        public TStatus CurrentStatus     { get; set; }

        /// <summary>
        /// Indicates if the operations is suspended
        /// </summary>
        public abstract bool IsSuspended { get; }
        /// <summary>
        /// Indicates if the operations is paused
        /// </summary>
        public abstract bool IsPaused    { get; }
        /// <summary>
        /// Indicates if the operations is completed
        /// </summary>
        public abstract bool IsCompleted { get; }
        /// <summary>
        /// Indicates if the number of retries have run out while the operation is still uncompleted
        /// </summary>
        public abstract bool IsTimedOut  { get; }
        /// <summary>
        /// Indicates if the operations is failed
        /// </summary>
        public abstract bool IsFailed    { get; }
    }
}
