﻿using System;

// http://dotnetfollower.com
namespace dotNetFollower
{
    /// <summary>
    /// Represents current state of a Bulk Delete operation
    /// </summary>
    [Serializable]
    public class BulkDeleteState : AsyncOperationState<BulkDeleteOperationState?, bulkdeleteoperation_statuscode?>
    {
        /// <summary>
        /// Id of the async job associated with the target Bulk Delete operation
        /// </summary>
        public Guid  AsyncJobId            { get; set; }

        /// <summary>
        /// Id of the target Bulk Delete operation. Null when the operation isn't created.
        /// </summary>
        public Guid? BulkDeleteOperationId { get; set; }

        /// <summary>
        /// Current number of successfully deleted records
        /// </summary>
        public int?  SuccessCount          { get; set; }

        /// <summary>
        /// Current number of problem records
        /// </summary>
        public int?  FailureCount          { get; set; }

        /// <summary>
        /// Indicates if the Bulk Delete is suspended
        /// </summary>
        public override bool IsSuspended
        {
            get { return CurrentState != null && CurrentState.Value == BulkDeleteOperationState.Suspended; }
        }
        /// <summary>
        /// Indicates if the Bulk Delete is paused
        /// </summary>
        public override bool IsPaused
        {
            get { return CurrentState != null && CurrentState.Value == BulkDeleteOperationState.Locked && CurrentStatus != null && CurrentStatus.Value == bulkdeleteoperation_statuscode.Pausing; }
        }
        /// <summary>
        /// Indicates if the Bulk Delete is completed
        /// </summary>
        public override bool IsCompleted
        {
            get { return CurrentState != null && CurrentState.Value == BulkDeleteOperationState.Completed; }
        }
        /// <summary>
        /// Indicates if the number of retries have run out while the Bulk Delete is still uncompleted
        /// </summary>
        public override bool IsTimedOut
        {
            get { return !IsCompleted && CurrentRetryCount <= 0; }
        }
        /// <summary>
        /// Indicates if the Bulk Delete is failed
        /// </summary>
        public override bool IsFailed
        {
            get { return CurrentStatus != null && CurrentStatus.Value == bulkdeleteoperation_statuscode.Failed; }
        }
        /// <summary>
        /// Indicates if the Bulk Delete is created
        /// </summary>
        public bool IsCreated
        {
            get { return BulkDeleteOperationId != null && BulkDeleteOperationId.Value != Guid.Empty; }
        }
    }
}
