﻿using System;
using System.Text;
using Microsoft.Crm.Sdk.Samples;
using Microsoft.Xrm.Sdk.Client;

// http://dotnetfollower.com
namespace dotNetFollower
{
    class Program
    {
        private const string SampleCrmConfiguration = @"<?xml version=""1.0"" encoding=""utf-8""?>
          <Configurations>
          <Configuration>
            <ServerAddress>MyCrmOnPremise</ServerAddress>
            <OrganizationName>My Company Friendly Name</OrganizationName>
            <DiscoveryUri>http://MyCrmOnPremise/XRMServices/2011/Discovery.svc</DiscoveryUri>
            <OrganizationUri>http://MyCrmOnPremise/MyCompanyFriendlyName/XRMServices/2011/Organization.svc</OrganizationUri>
            <HomeRealmUri />
            <Credentials>
              <UserName>dotNetFollower</UserName>
              <Domain>hq</Domain>
              <Password>11111</Password>
            </Credentials>
            <EndpointType>ActiveDirectory</EndpointType>
            <UserPrincipalName />
          </Configuration>
        </Configurations>";

        static void Main(string[] args)
        {
            try
            {
                //ConnectUsingServerConnection();
                //ConnectUsingCustomServerConnection();
                ConnectUsingCustomServerConnection2();
            }
            catch (Exception ex)
            {
                WriteError(ex);
            }
        }

        public static void ConnectUsingServerConnection()
        {
            // Obtain the target organization's Web address and client logon credentials from the user.
            var serverConnect = new ServerConnection();
            ServerConnection.Configuration config = serverConnect.GetServerConfiguration();

            // Connect to the Organization service. 
            using (OrganizationServiceProxy serviceProxy = ServerConnection.GetOrganizationProxy(config))
            {
                // This statement is required to enable early-bound type support.
                serviceProxy.EnableProxyTypes();

                // do something useful
            }
        }

        public static void ConnectUsingCustomServerConnection()
        {
            // Obtain the target organization's Web address and client logon credentials from the xml string.
            var serverConnection = new ServerConnectionEx();
            ServerConnection.Configuration configuration = serverConnection.Parse(SampleCrmConfiguration);
            
            // Connect to the Organization service. 
            using (OrganizationServiceProxy serviceProxy = ServerConnection.GetOrganizationProxy(configuration))
            {
                // This statement is required to enable early-bound type support.
                serviceProxy.EnableProxyTypes();

                // do something useful
            }
        }

        public static void ConnectUsingCustomServerConnection2()
        {
            var serverConnection = new ServerConnectionEx();
            serverConnection.DoInOrganizationServiceProxyContext(SampleCrmConfiguration, proxy =>
            {
                Guid userId = proxy.CurrentUser.UserId;
                
                // do something else
            });
        }

        static void WriteError(Exception ex)
        {
            var sb = new StringBuilder(ex.Message);
            if (ex.InnerException != null)
                sb.Append(" [InnerException: ").Append(ex.InnerException.Message).Append("]");
            Console.Write(sb.ToString());
        }
    }
}
